import xlrd
from datetime import datetime, date

ALLOWED_EXTENSIONS = ['xls', 'xlsx']


def allowed_file(filename):
    return '.' in filename and filename.rsplit('.', 1)[1] in ALLOWED_EXTENSIONS


def is_int(num):
    # print(num)
    # print(type(num))
    if isinstance(num, float):
        n = num * 10 % 10
        if n == 0.0:
            return True
        else:
            return False
    elif isinstance(num, int):
        return True
    else:
        return False


def excel_no(data=None):
    if isinstance(data, str):
        return data
    elif isinstance(data, float):
        return str(int(data))
    else:
        return None


def json_serial(obj):
    """JSON serializer for objects not serializable by default json code"""

    if isinstance(obj, datetime):
        return obj.strftime('%Y-%m-%d %H:%M:%S')
    elif isinstance(obj, date):
        return obj.strftime('%Y-%m-%d')
    raise TypeError("Type %s not serializable" % type(obj))


def excel_date_to_db(data=None):
    if isinstance(data, str):
        try:
            d = datetime.strptime(data, '%Y-%m-%d %H:%M:%S') if data else None
            return d
        except Exception:
            return None
    try:
        datetime_data = xlrd.xldate_as_datetime(data, 0) if data else None
        return datetime_data
    except Exception:
        return None


def excel_no_to_db(data=None):
    if isinstance(data, str):
        return data
    elif isinstance(data, float):
        return str(int(data))
    elif isinstance(data, int):
        return str(data)
    else:
        return None



from app import hsexcel
from app import checker
from .checker import ErrorChecker

celery = hsexcel.make_celery()


def get_checker(excel_type):
    return checker.get(excel_type, ErrorChecker)


@celery.task
def check_error(id=None, param=None, error_path=None):
    excel_type = param.get('excel_type')[0]
    Checker = get_checker(excel_type)
    checker = Checker(id, error_path, param)
    checker.as_json()
    checker.add_error()
    return 'success'



from marshmallow import Schema, fields


class ExcleSchema(Schema):
    excel_id = fields.Integer()
    name = fields.String()
    import_time = fields.DateTime(format='%Y-%m-%d %H:%M:%S')
    status = fields.Integer()
    excel_type = fields.String()
    extend = fields.String()

class BaseSchema(Schema):
    # 自定义Schema
    def _str(self, data=None):
        if isinstance(data, str):
            return data
        elif isinstance(data, float):
            return str(int(data))
        else:
            return None

    def list_to_str(self, data=None):
        if data:
            return str(data)


class StyleSchema(BaseSchema):
    item = fields.Method(deserialize="_str", attribute='size_code')
    model = fields.Method(deserialize="_str", attribute='color_code')
    iman = fields.Method(deserialize="_str", attribute='style_code')
    color = fields.Method(deserialize="_str", attribute='color_name')
    scale = fields.Method(deserialize="_str", attribute='size_name')
    seq = fields.Method(deserialize="_str", attribute='order')
    pcb = fields.Integer(attribute='per_box_case')
    ue = fields.Integer(attribute='per_tape_case')
    box = fields.Method(deserialize="_str", attribute='box_spec')
    season = fields.Method(deserialize="_str", attribute='season_name')
    brand = fields.Method(deserialize="_str", attribute='brand_name')
    error = fields.Method(deserialize="list_to_str")


class MaterialSchema(BaseSchema):
    materialname = fields.Method(deserialize="_str", attribute='material_name')  # 物种名称
    mmodelcode = fields.Method(deserialize="_str", attribute='material_code')  # 物种编码
    stockunit = fields.Method(deserialize="_str", attribute='stock_unit')  # 库存单位
    pruchaseunit = fields.Method(deserialize="_str", attribute='pr_unit')  # 采购单位
    rate = fields.Float(attribute='unit_stock2pr_rate')  # 采购库存转换率
    specification = fields.Method(deserialize="_str", attribute='spec')  # 规格型号
    component = fields.Method(deserialize="_str", attribute='component')  # 成分
    gmwidth = fields.Method(deserialize="_str", attribute='grey_fabric_width')  # 坯布幅宽
    gmm = fields.Method(deserialize="_str", attribute='grey_fabric_weight')  # 坯布克重
    fpwidth = fields.Method(deserialize="_str", attribute='finish_product_width')  # 成品幅宽
    fpm = fields.Method(deserialize="_str", attribute='finish_product_weight')  # 成品克重
    scale = fields.Method(deserialize="_str", attribute='scale')  # 辅料第一规格
    secondscale = fields.Method(deserialize="_str", attribute='second_scale')  # 辅料第二规格
    mcolorname = fields.Method(deserialize="_str", attribute='material_item_name')  # 颜色名称
    mcolorenname = fields.Method(deserialize="_str", attribute='material_item_en_name')
    mcolorcode = fields.Method(deserialize="_str", attribute='material_item_code')  # 色号，颜色编码(相同物料下唯一)
    mitemcode = fields.Method(deserialize="_str", attribute='material_item_no')  # 物料编码(全局唯一)


class InventorySchema(Schema):
    material_item_no = fields.String()  # 物料编码(model)
    material_item_code = fields.String()  # 物料色号(item)
    material_item_spec = fields.String()  # 物料描述
    production_line = fields.String()  # 生产线
    storage_quantity = fields.Number(as_string=True)  # 在库数
    unit_name = fields.String()  # 单位
    remark = fields.String()  # 备注
    error = fields.String()
    excel_id = fields.String()


class InventoryStyleSchema(Schema):
    style_code = fields.String()  # 款号
    color_code = fields.String()  # 色号(model编码)
    size_group_code = fields.String()  # 尺码组编码
    size_item_code = fields.String()  # 尺码编码
    storage_quantity = fields.Number(as_string=True)  # 在库数
    size_name = fields.String()  # 尺码名称
    # remark = fields.String()
    error = fields.String()
    excel_id = fields.String()


class BomSchema(Schema):
    pass



import sys, os, logging
from .views import hs_excel
from celery import Celery
from .checker import ErrorChecker
from .utils import excel_no_to_db, excel_date_to_db
from .models import ExcelModel

logger = logging.getLogger(__name__)


class HsExcel(object):

    def __init__(self, app=None, checker=None):
        if app is not None:
            self.app = app
            self.checker = checker
            # self.db = db
            self.init_app(self.app, self.checker)
        else:
            self.app = None
            # self.db = None

    def init_app(self, app, checker):
        self.app = app
        self.checker = checker
        excel_path = self.app.config.get('EXCEL_PATH')
        # 判断是否传入　excelpath
        if not excel_path:
            logger.error("EXCEL_PATH没有配置")
            sys.exit(0)
        # 判断checker是否正确
        if not isinstance(self.checker, dict):
            logger.error('checker不是一个dict')
            sys.exit(0)
        if isinstance(self.checker, dict):
            for value in self.checker.values():
                if not isinstance(value, type):
                    logger.error('checker配置错误')
                    sys.exit(0)
                if not issubclass(value, ErrorChecker):
                    logger.error('checker配置不正确')
                    sys.exit(0)
        import_path = os.path.join(excel_path, 'import')
        error_path = os.path.join(excel_path, 'error')
        for _dir in [excel_path, import_path, error_path]:
            if not os.path.exists(_dir):
                os.mkdir(_dir)
        # 注册路由
        self.app.register_blueprint(hs_excel, url_prefix='/hs_excel')

    def make_celery(self):
        celery = Celery(self.app.import_name, broker=self.app.config['CELERY_BROKER_URL'])
        celery.conf.update(self.app.config)
        return celery

    def print_test(self):

        # print('hhhh')
        return '这是插件里面的内容'


__all__ = [HsExcel, excel_date_to_db, excel_no_to_db, ExcelModel, ErrorChecker]



from setuptools import setup

setup(
    name='Flask-HSExcel',
    version='0.1.2',
    license='MIT',
    author='Alan Zhang',
    author_email='1095087479@qq.com',
    description=u'环思的flask导入excel的插件',
    long_description=__doc__,
    packages=['flask_hsexcel'],
    zip_safe=False,
    include_package_data=True,
    platforms='any',
    install_requires=[
        'Flask',
        'SQLAlchemy',
        'celery',
        'flask-DouWa',
        'Flask-SQLAlchemy',
        'marshmallow',
        'xlrd',
        'xlutils',
        'xlwt',
        'openpxxl'
    ],
    classifiers=[
        'Framework :: Flask',
        'Natural Language :: English',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ]
)
